//==============================================================================
// King of the Unknown
// by Felix Hermansson, Tsunami Studios (hoodncloak@hotmail.com)
// 18 January 2006
//
// King of the Hill style Random Map Script for Age of Empires III
//==============================================================================


//==============================================================================
// includes
//==============================================================================

// include mercenaries
include "mercenaries.xs";


//==============================================================================
// function main 
// generates map
//==============================================================================

void main(void)
{
   // Map generation status bar
   rmSetStatusText("",0.01);

   // Check for nomad start (additional buildings)
   bool nomad = rmGetNomadStart();

   // Define map size
   int playerTiles = 20000;
   if (cNumberNonGaiaPlayers >4)
   {
      playerTiles = 16000;
   }
   int size = 2.0 * sqrt(cNumberNonGaiaPlayers * playerTiles);
   rmSetMapSize(size, size);  

   // Choose basic map skin (snow, grass or desert)
   int mapSkin = rmRandInt(1,3);

   // Pick default terrain, set lighting, and set weather 
   if (mapSkin == 1)
   {
      rmEnableLocalWater(false);
      rmSetBaseTerrainMix("greatlakes_snow");
      rmTerrainInitialize("great_lakes\ground_ice1_gl", 1.0);
      rmSetMapType("greatlakes");
      rmSetMapType("land");
      rmSetMapType("snow");
      rmSetLightingSet("Great Lakes Winter");
      rmSetWindMagnitude(3.0);
      rmSetGlobalSnow(1.0);
   }
   else if (mapSkin == 2)
   {
      rmEnableLocalWater(false);
      rmSetBaseTerrainMix("greatlakes_grass");
      rmTerrainInitialize("great_plains\ground5_gp", 1.0);
      rmSetMapType("greatlakes");
      rmSetMapType("land");
      rmSetMapType("grass");
      rmSetLightingSet("Great Lakes");
      rmSetWindMagnitude(1.0);
   }
   else // mapSkin == 3
   {
      rmEnableLocalWater(false);
      rmSetBaseTerrainMix("texas_dirt");
      rmTerrainInitialize("texas\ground1_tex", 5.0);
      rmSetMapType("texas");
      rmSetMapType("land");
      rmSetMapType("grass");
      rmSetLightingSet("Texas");
      rmSetWindMagnitude(3.0);
   }

   // Set map elevation parameters
   rmSetMapElevationParameters(cElevTurbulence, 0.09, 6, 0.4, 7.0);
   rmSetMapElevationHeightBlend(1);

   // Choose mercenaries
   chooseMercs();

   // Map generation status bar
   rmSetStatusText("",0.10);


   // Define constraints to have objects and areas avoid each other

   // Define necessary classes
   int classPlayer = rmDefineClass("player");
   rmDefineClass("classCliff");
   rmDefineClass("classLake");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("importantItem");
   rmDefineClass("natives");
   rmDefineClass("nuggets");
   rmDefineClass("center");

   // Corner constraint
   rmSetWorldCircleConstraint(true);
   
   // Map edge constraints
   int playerEdgeConstraint = rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(10), rmZTilesToFraction(10), 1.0-rmXTilesToFraction(10), 1.0-rmZTilesToFraction(10), 0.01);
   int longPlayerEdgeConstraint = rmCreateBoxConstraint("long avoid edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);
   
   // Center constraints
   int centerConstraint = rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), rmXFractionToMeters(0.10));
   int centerConstraintFar = rmCreateClassDistanceConstraint("stay away from center far", rmClassID("center"), rmXFractionToMeters(0.23));
   int centerConstraintExtreme = rmCreateClassDistanceConstraint("stay away from center extremely far", rmClassID("center"), rmXFractionToMeters(0.50));

   // General circle constraint
   int circleConstraint = rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Cardinal direction constraints
   int Northward = rmCreatePieConstraint("northMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(315), rmDegreesToRadians(135));
   int Southward = rmCreatePieConstraint("southMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(135), rmDegreesToRadians(315));
   int Eastward = rmCreatePieConstraint("eastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(45), rmDegreesToRadians(225));
   int Westward = rmCreatePieConstraint("westMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(225), rmDegreesToRadians(45));

   // Map quadrant constraints
   int Northeast = rmCreatePieConstraint("northeastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(45), rmDegreesToRadians(135));
   int Southeast = rmCreatePieConstraint("southeastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(135), rmDegreesToRadians(225));
   int Southwest = rmCreatePieConstraint("southwestMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(225), rmDegreesToRadians(315));
   int Northwest = rmCreatePieConstraint("northwestMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(315), rmDegreesToRadians(45));

   // Player constraints
   int extremePlayerConstraint = rmCreateClassDistanceConstraint("extreme distance player constraint", classPlayer, 100.0);
   int longPlayerConstraint = rmCreateClassDistanceConstraint("long distance player constraint", classPlayer, 70.0);
   int playerConstraint = rmCreateClassDistanceConstraint("standard distance player constraint", classPlayer, 45.0);
   int shortPlayerConstraint = rmCreateClassDistanceConstraint("short distance player constraint", classPlayer, 20.0);

   // Nature constraints (forests, mines & starting resources)
   int forestConstraint = rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 40.0);
   int forestConstraintShort = rmCreateClassDistanceConstraint("avoid forest", rmClassID("classForest"), 10.0);
   int avoidCoin = rmCreateTypeDistanceConstraint("avoid coin", "mine", 40.0);
   int shortAvoidCoin = rmCreateTypeDistanceConstraint("short avoid coin", "mine", 10.0);
   int avoidResource = rmCreateTypeDistanceConstraint("resource avoid resource", "resource", 20.0);
   int avoidStartResource = rmCreateTypeDistanceConstraint("start resource no overlap", "resource", 10.0);

   // Impassable land constraints
   int avoidImpassableLand = rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);
   int shortAvoidImpassableLand = rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 2.0);
   int longAvoidImpassableLand = rmCreateTerrainDistanceConstraint("long avoid impassable land", "Land", false, 10.0);

   // Cliff constraints
   int avoidCliffs = rmCreateClassDistanceConstraint("cliff vs. cliff", rmClassID("classCliff"), 40.0);
   int avoidCliffsShort = rmCreateClassDistanceConstraint("short avoid cliff", rmClassID("classCliff"), 10.0);

   // Lake constraints
   int avoidLakesFar = rmCreateClassDistanceConstraint("far avoid lake", rmClassID("classLake"), 150.0);
   int avoidLakes = rmCreateClassDistanceConstraint("lake vs. lake", rmClassID("classLake"), 50.0);
   int avoidLakesShort = rmCreateClassDistanceConstraint("short avoid lake", rmClassID("classLake"), 10.0);

   // Unit constraints
   int avoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 45.0);
   int shortAvoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units short", rmClassID("startingUnit"), 10.0);
   int avoidImportantItem = rmCreateClassDistanceConstraint("general items avoid each other", rmClassID("importantItem"), 10.0);
   int avoidNatives = rmCreateClassDistanceConstraint("objects avoid natives", rmClassID("natives"), 30.0);
   int avoidNativesLong = rmCreateClassDistanceConstraint("objects avoid natives long", rmClassID("natives"), 70.0);
   int avoidNativesExtreme = rmCreateClassDistanceConstraint("objects avoid natives extreme", rmClassID("natives"), 100.0);
   int avoidNuggets = rmCreateClassDistanceConstraint("objects avoid nuggets", rmClassID("nuggets"), 60.0);
   int shortNuggetConstraint = rmCreateTypeDistanceConstraint("avoid nugget objects", "AbstractNugget", 7.0);
   int muskoxConstraint = rmCreateTypeDistanceConstraint("avoid musk oxen", "muskOx", 40.0);
   int caribouConstraint = rmCreateTypeDistanceConstraint("avoid caribous", "caribou", 40.0);
   int elkConstraint = rmCreateTypeDistanceConstraint("avoid elks", "elk", 40.0);
   int deerConstraint = rmCreateTypeDistanceConstraint("avoid deer", "deer", 40.0);
   int bisonConstraint = rmCreateTypeDistanceConstraint("avoid bisons", "bison", 40.0);
   int pronghornConstraint = rmCreateTypeDistanceConstraint("avoid pronghorns", "pronghorn", 40.0);
   int sheepConstraint = rmCreateTypeDistanceConstraint("avoid sheep", "sheep", 40.0);
   int cowConstraint = rmCreateTypeDistanceConstraint("avoid cows", "cow", 40.0);
   int berryConstraint = rmCreateTypeDistanceConstraint("avoid berrybushes", "berrybush", 40.0);

   // Additional building constraints
   int avoidOutposts = rmCreateTypeDistanceConstraint("avoid outposts", "outpost", 25.0);
   int avoidBlockhouses = rmCreateTypeDistanceConstraint("avoid blockhouses", "blockhouse", 35.0);

   // Decoration constraints
   int avoidVultures = rmCreateTypeDistanceConstraint("avoid vultures", "PropVulturePerching", 100.0);
   int avoidRock = rmCreateTypeDistanceConstraint("avoid rocks", "BigPropTexas", 100.0);

   // General constraint
   int avoidAll = rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // Trade route constraints
   int avoidTradeRoute = rmCreateTradeRouteDistanceConstraint("trade route", 5.0);
   int shortAvoidTradeRoute = rmCreateTradeRouteDistanceConstraint("short trade route", 3.0);
   int avoidTradeRouteFar = rmCreateTradeRouteDistanceConstraint("trade route far", 20.0);
   int avoidTradeSockets = rmCreateTypeDistanceConstraint("avoid trade sockets", "sockettraderoute", 10.0);
   int farAvoidTradeSockets = rmCreateTypeDistanceConstraint("far avoid trade sockets", "sockettraderoute", 40.0);
   int extremeAvoidTradeSockets = rmCreateTypeDistanceConstraint("extreme avoid trade sockets", "sockettraderoute", 80.0);
   int tradeRouteCircleConstraint = rmCreatePieConstraint("trade route circle onstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.1), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Map generation status bar
   rmSetStatusText("",0.20);


   // Create central trade route

   // Define sockets
   int socketID = rmCreateObjectDef("sockets to dock Trade Posts");
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 12.0);
   rmAddObjectDefConstraint(socketID, shortAvoidTradeRoute);
   rmAddObjectDefConstraint(socketID, avoidImpassableLand);
   rmAddObjectDefConstraint(socketID, tradeRouteCircleConstraint);

   // Create dirt route
   int tradeRouteID = rmCreateTradeRoute();
   rmAddTradeRouteWaypoint(tradeRouteID, 0.50, 0.60);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.55, 0.59, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.59, 0.55, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.60, 0.50, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.59, 0.45, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.55, 0.41, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.50, 0.40, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.45, 0.41, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.41, 0.45, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.40, 0.50, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.41, 0.55, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.45, 0.59, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.50, 0.60, 2, 5);
   rmBuildTradeRoute(tradeRouteID, "dirt");

   // Add 4 or 6 TP sockets along the trade route
   vector socketLoc = cOriginVector;
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);
   if (cNumberNonGaiaPlayers < 5)
   {
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.35);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.6);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.85);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   }
   else
   {
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.27);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.43);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.6);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.77);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
      socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.93);
      rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   }

   // Map generation status bar
   rmSetStatusText("",0.30);


   // Place players

   // Place players in a rough circle on the outskirts of the map
   rmSetTeamSpacingModifier(0.7);
   rmPlacePlayersCircular(0.40, 0.45, rmDegreesToRadians(360/(cNumberNonGaiaPlayers*3)));

   // Create player areas
   float playerFraction = rmAreaTilesToFraction(200);
   for(i=1; <cNumberPlayers)
   {
      int id = rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmSetAreaCoherence(id, 1.0);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      rmAddAreaConstraint(id, playerConstraint); 
      rmAddAreaConstraint(id, playerEdgeConstraint);
      rmAddAreaConstraint(id, longAvoidImpassableLand);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build all player areas
   rmBuildAllAreas();

   // Define town center (nomad start ignored here)
   int startingTCID =  rmCreateObjectDef("startingTC");
   rmAddObjectDefItem(startingTCID, "townCenter", 1, 5.0);
   rmAddObjectDefConstraint(startingTCID, avoidImpassableLand);
   rmAddObjectDefConstraint(startingTCID, avoidTradeRoute);
   rmAddObjectDefConstraint(startingTCID, avoidTradeSockets);
   rmSetObjectDefMinDistance(startingTCID, 0.0);
   rmSetObjectDefMaxDistance(startingTCID, 0.0);
   rmAddObjectDefToClass(startingTCID, rmClassID("player"));

   // Define additional buildings for 'nomad' game
   if (nomad == true)
   {
      // Starting barracks
      int startingBarracks = rmCreateObjectDef("starting barracks");
      rmAddObjectDefItem(startingBarracks, "barracks", 1, 5.0);
      rmSetObjectDefMinDistance(startingBarracks, 20.0);
      rmSetObjectDefMaxDistance(startingBarracks, 22.0);
      rmAddObjectDefToClass(startingBarracks, rmClassID("player"));
      rmAddObjectDefToClass(startingBarracks, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingBarracks, avoidOutposts);
      rmAddObjectDefConstraint(startingBarracks, avoidImportantItem);

      // Starting outpost
      int startingOutpost = rmCreateObjectDef("starting outpost");
      rmAddObjectDefItem(startingOutpost, "outpost", 1, 5.0);
      rmSetObjectDefMinDistance(startingOutpost, 20.0);
      rmSetObjectDefMaxDistance(startingOutpost, 22.0);
      rmAddObjectDefToClass(startingOutpost, rmClassID("player"));
      rmAddObjectDefToClass(startingOutpost, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingOutpost, avoidOutposts);
      rmAddObjectDefConstraint(startingOutpost, avoidImportantItem);

      // Starting blockhouse
      int startingBlockhouse = rmCreateObjectDef("starting blockhouse");
      rmAddObjectDefItem(startingBlockhouse, "blockhouse", 1, 5.0);
      rmSetObjectDefMinDistance(startingBlockhouse, 20.0);
      rmSetObjectDefMaxDistance(startingBlockhouse, 22.0);
      rmAddObjectDefToClass(startingBlockhouse, rmClassID("player"));
      rmAddObjectDefToClass(startingBlockhouse, rmClassID("importantItem"));
      rmAddObjectDefConstraint(startingBlockhouse, avoidBlockhouses);
      rmAddObjectDefConstraint(startingBlockhouse, avoidImportantItem);
   }

   // Define starting units (explorer, envoy, native scout & war dog)
   int startingUnits = rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 10.0);
   rmSetObjectDefMaxDistance(startingUnits, 12.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));

   // Define starting mine close
   int startSilver1ID = rmCreateObjectDef("player silver close");
   rmAddObjectDefItem(startSilver1ID, "mine", 1, 5.0);
   rmSetObjectDefMinDistance(startSilver1ID, 12.0);
   rmSetObjectDefMaxDistance(startSilver1ID, 20.0);
   rmAddObjectDefConstraint(startSilver1ID, avoidStartResource);
   rmAddObjectDefConstraint(startSilver1ID, avoidTradeRoute);
   rmAddObjectDefConstraint(startSilver1ID, avoidTradeSockets);
   rmAddObjectDefConstraint(startSilver1ID, avoidImpassableLand);

   // Define starting mine medium
   int startSilver2ID = rmCreateObjectDef("player silver medium");
   rmAddObjectDefItem(startSilver2ID, "mine", 1, 5.0);
   rmSetObjectDefMinDistance(startSilver2ID, 45.0);
   rmSetObjectDefMaxDistance(startSilver2ID, 50.0);
   rmAddObjectDefConstraint(startSilver2ID, playerConstraint);
   rmAddObjectDefConstraint(startSilver2ID, avoidTradeRoute);
   rmAddObjectDefConstraint(startSilver2ID, avoidTradeSockets);
   rmAddObjectDefConstraint(startSilver2ID, avoidImpassableLand);

   // Define starting mine far
   int startSilver3ID = rmCreateObjectDef("player silver far");
   rmAddObjectDefItem(startSilver3ID, "mine", 1, 5.0);
   rmSetObjectDefMinDistance(startSilver3ID, 60.0);
   rmSetObjectDefMaxDistance(startSilver3ID, 80.0);
   rmAddObjectDefConstraint(startSilver3ID, longPlayerConstraint);
   rmAddObjectDefConstraint(startSilver3ID, avoidTradeRoute);
   rmAddObjectDefConstraint(startSilver3ID, avoidTradeSockets);
   rmAddObjectDefConstraint(startSilver3ID, avoidImpassableLand);

   // Define starting food close
   int startFood1ID = rmCreateObjectDef("player food close");
   if (mapSkin == 1) // snow
   {
      rmAddObjectDefItem(startFood1ID, "caribou", rmRandInt(3,5), 4.0);
   }
   else if (mapSkin == 2) // grass
   {
      rmAddObjectDefItem(startFood1ID, "berrybush", rmRandInt(3,4), 2.0);
   }
   else // desert
   {
      rmAddObjectDefItem(startFood1ID, "berrybush", rmRandInt(2,3), 2.0);
   }
   rmSetObjectDefMinDistance(startFood1ID, 6.0);
   rmSetObjectDefMaxDistance(startFood1ID, 12.0);
   rmAddObjectDefConstraint(startFood1ID, avoidStartResource);
   rmAddObjectDefConstraint(startFood1ID, avoidTradeRoute);
   rmAddObjectDefConstraint(startFood1ID, avoidTradeSockets);
   rmAddObjectDefConstraint(startFood1ID, avoidImpassableLand);
   rmSetObjectDefCreateHerd(startFood1ID, true);

   // Define starting food medium
   int startFood2ID = rmCreateObjectDef("player food medium");
   if (mapSkin == 1) // snow
   {
      rmAddObjectDefItem(startFood2ID, "muskOx", rmRandInt(5,7), 5.0);
   }
   else if (mapSkin == 2) // grass
   {
      rmAddObjectDefItem(startFood2ID, "deer", rmRandInt(6,8), 5.0);
   }
   else // desert
   {
      rmAddObjectDefItem(startFood2ID, "pronghorn", rmRandInt(4,6), 5.0);
   }
   rmSetObjectDefMinDistance(startFood2ID, 24.0);
   rmSetObjectDefMaxDistance(startFood2ID, 36.0);
   rmAddObjectDefConstraint(startFood2ID, avoidStartResource);
   rmAddObjectDefConstraint(startFood2ID, avoidTradeRoute);
   rmAddObjectDefConstraint(startFood2ID, avoidImpassableLand);
   rmSetObjectDefCreateHerd(startFood2ID, true);

   // Define starting trees
   int startAreaTreeID = rmCreateObjectDef("starting trees");
   if (mapSkin == 1) // snow
   {
      rmAddObjectDefItem(startAreaTreeID, "TreeGreatLakesSnow", rmRandInt(3,4), 6.0);
   }
   else if (mapSkin == 2) // grass
   {
      rmAddObjectDefItem(startAreaTreeID, "TreeGreatPlains", rmRandInt(4,5), 6.0);
   }
   else // desert
   {
      rmAddObjectDefItem(startAreaTreeID, "TreeTexasDirt", rmRandInt(2,3), 6.0);
   }
   rmSetObjectDefMinDistance(startAreaTreeID, 12);
   rmSetObjectDefMaxDistance(startAreaTreeID, 16);
   rmAddObjectDefConstraint(startAreaTreeID, avoidStartResource);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(startAreaTreeID, avoidTradeRoute);
   rmAddObjectDefConstraint(startAreaTreeID, avoidTradeSockets);
   rmAddObjectDefConstraint(startAreaTreeID, avoidNatives);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidStartingUnits);

   // Place starting units and resources for all payers 
   for(i=1; <cNumberPlayers)
   {
      rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      if (nomad == true)
      {
         if (rmGetPlayerCiv(i) == rmGetCivID("Russians"))
         {
            rmPlaceObjectDefAtLoc(startingBlockhouse, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingBlockhouse, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         }
         else
         {
            rmPlaceObjectDefAtLoc(startingOutpost, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingOutpost, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
            rmPlaceObjectDefAtLoc(startingBarracks, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
         }
      }
      rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilver1ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilver2ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilver3ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startFood1ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startFood2ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Map generation status bar
   rmSetStatusText("",0.40);


   // Place lakes and cliffs

   // Define area for lakes
   int lakeAreaId = rmCreateArea("lake area");
   rmSetAreaSize(lakeAreaId, 0.5, 0.5);
   rmSetAreaLocation(lakeAreaId, 0.5, 0.5);
   rmSetAreaCoherence(lakeAreaId, 1.0);
   rmSetAreaWarnFailure(lakeAreaId, false);
   rmBuildArea(lakeAreaId);

   // Define and place lakes
   int numLakes = 0;
   if (mapSkin == 1) // snow
   {
      numLakes = rmRandInt(0,cNumberNonGaiaPlayers);
   }
   else if (mapSkin == 2) // grass
   {
      numLakes = rmRandInt(0,(cNumberNonGaiaPlayers + 2));
   }
   else // desert
   {
      numLakes = rmRandInt(0,1);
   }
   for (i=0; <numLakes)
   {
      int lakeID = rmCreateArea("lake"+i, lakeAreaId);
      if (mapSkin == 1) // snow
      {
         rmSetAreaSize(lakeID, rmAreaTilesToFraction(400), rmAreaTilesToFraction(600));
         rmSetAreaWaterType(lakeID, "great lakes ice");
      }
      else if (mapSkin == 2) // grass
      {
         rmSetAreaSize(lakeID, rmAreaTilesToFraction(1000/numLakes), rmAreaTilesToFraction(800));
         rmSetAreaWaterType(lakeID, "great plains pond");
      }
      else // desert
      {
         rmSetAreaSize(lakeID, rmAreaTilesToFraction(300), rmAreaTilesToFraction(500));
         rmSetAreaWaterType(lakeID, "texas pond");
      }
      rmSetAreaWarnFailure(lakeID, false);
      rmSetAreaBaseHeight(lakeID, 1.0);
      rmSetAreaObeyWorldCircleConstraint(lakeID, false);
      rmAddAreaToClass(lakeID, rmClassID("classLake"));
      rmAddAreaConstraint(lakeID, avoidImportantItem);
      rmAddAreaConstraint(lakeID, avoidImpassableLand);
      rmAddAreaConstraint(lakeID, avoidLakes);
      rmAddAreaConstraint(lakeID, extremePlayerConstraint);
      rmAddAreaConstraint(lakeID, avoidStartingUnits);
      rmAddAreaConstraint(lakeID, avoidTradeRouteFar);
      rmAddAreaConstraint(lakeID, avoidNatives);
      rmSetAreaMinBlobs(lakeID, 2);
      rmSetAreaMaxBlobs(lakeID, 3);
      rmSetAreaMinBlobDistance(lakeID, 5.0);
      rmSetAreaMaxBlobDistance(lakeID, 20.0);
      rmSetAreaSmoothDistance(lakeID, 5);
      rmSetAreaCoherence(lakeID, 0.5);
      rmBuildArea(lakeID);
   }

   // Define and place cliffs
   int numCliffs = 0;
   if (mapSkin == 1) // snow
   {
      numCliffs = rmRandInt(1,cNumberNonGaiaPlayers) + rmRandInt(1,cNumberNonGaiaPlayers);
   }
   else if (mapSkin == 2) // grass
   {
      numCliffs = rmRandInt(0,cNumberNonGaiaPlayers) + rmRandInt(0,cNumberNonGaiaPlayers);
   }
   else // desert
   {
      numCliffs = cNumberNonGaiaPlayers * rmRandInt(4,6);
   }
   for (i=0; <numCliffs)
   {
      int cliffID = rmCreateArea("cliff"+i);
      rmSetAreaSize(cliffID, rmAreaTilesToFraction(100), rmAreaTilesToFraction(500));
      rmSetAreaWarnFailure(cliffID, false);
      if (mapSkin == 1) // snow
      {
         rmSetAreaCliffType(cliffID, "rocky mountain edge");
         rmSetAreaCliffEdge(cliffID, 2, 0.3, 0.1, 1.0, 0);
         rmSetAreaCliffPainting(cliffID, false, true, true, 1.5, true);
         rmSetAreaCliffHeight(cliffID, 5, 2.0, 0.5); 
         rmSetAreaHeightBlend(cliffID, 1);
      }
      else if (mapSkin == 2) // grass
      {
         rmSetAreaCliffType(cliffID, "New England");
         rmSetAreaCliffEdge(cliffID, 2, 0.3, 0.1, 1.0, 0);
         rmSetAreaCliffPainting(cliffID, false, true, true, 1.5, true);
         rmSetAreaCliffHeight(cliffID, 4, 1.0, 0.5); 
         rmSetAreaHeightBlend(cliffID, 1);
      }
      else // desert
      {
         rmSetAreaCliffType(cliffID, "Texas");
         rmSetAreaCliffEdge(cliffID, 1, 0.8, 0.2, 1.0, 0);
         rmSetAreaCliffPainting(cliffID, false, true, true, 1.5, true);
         rmSetAreaCliffHeight(cliffID, 0, 7.0, 0.5); 
         rmSetAreaHeightBlend(cliffID, 1);
      }
      rmSetAreaObeyWorldCircleConstraint(cliffID, false);
      rmAddAreaToClass(cliffID, rmClassID("classCliff")); 
      rmAddAreaConstraint(cliffID, avoidImportantItem);
      rmAddAreaConstraint(cliffID, avoidImpassableLand);
      rmAddAreaConstraint(cliffID, avoidLakesShort);
      rmAddAreaConstraint(cliffID, avoidCliffs);
      rmAddAreaConstraint(cliffID, extremePlayerConstraint);
      rmAddAreaConstraint(cliffID, avoidStartingUnits);
      rmAddAreaConstraint(cliffID, avoidTradeRouteFar);
      rmAddAreaConstraint(cliffID, avoidNatives);
      rmSetAreaMinBlobs(cliffID, 2);
      rmSetAreaMaxBlobs(cliffID, 4);
      rmSetAreaMinBlobDistance(cliffID, 5.0);
      rmSetAreaMaxBlobDistance(cliffID, 20.0);
      rmSetAreaSmoothDistance(cliffID, 5);
      rmSetAreaCoherence(cliffID, 0.5);
      rmBuildArea(cliffID);
   }

   // Map generation status bar
   rmSetStatusText("",0.50);


   // Place silver mines and forests

   // Place silver mines (North & South)
   int silverCount = 0;
   if (mapSkin == 1) // snow
   {
      silverCount = (cNumberNonGaiaPlayers * 2) + rmRandInt(0,(cNumberNonGaiaPlayers / 2));
   }
   else if (mapSkin == 2) // grass
   {
      silverCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
   }
   else // desert
   {
      silverCount = (cNumberNonGaiaPlayers * 3) - rmRandInt(0,(cNumberNonGaiaPlayers / 2));
   }
   // Northern mines
   int northernMineID = rmCreateObjectDef("northern mines");
   rmAddObjectDefItem(northernMineID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(northernMineID, 0.0);
   rmSetObjectDefMaxDistance(northernMineID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(northernMineID, avoidImpassableLand);
   rmAddObjectDefConstraint(northernMineID, avoidCoin);
   rmAddObjectDefConstraint(northernMineID, longPlayerConstraint);
   rmAddObjectDefConstraint(northernMineID, avoidAll);
   rmAddObjectDefConstraint(northernMineID, avoidTradeRoute);
   rmAddObjectDefConstraint(northernMineID, avoidTradeSockets);
   rmAddObjectDefConstraint(northernMineID, Northward);
   rmPlaceObjectDefAtLoc(northernMineID, 0, 0.5, 0.5);
   for(i=0; <silverCount)
   {
      rmPlaceObjectDefAtLoc(northernMineID, 0, 0.5, 0.5);
   }
   // Southern mines
   int southernMineID = rmCreateObjectDef("southern mines");
   rmAddObjectDefItem(southernMineID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(southernMineID, 0.0);
   rmSetObjectDefMaxDistance(southernMineID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(southernMineID, avoidImpassableLand);
   rmAddObjectDefConstraint(southernMineID, avoidCoin);
   rmAddObjectDefConstraint(southernMineID, longPlayerConstraint);
   rmAddObjectDefConstraint(southernMineID, avoidAll);
   rmAddObjectDefConstraint(southernMineID, avoidTradeRoute);
   rmAddObjectDefConstraint(southernMineID, avoidTradeSockets);
   rmAddObjectDefConstraint(southernMineID, Southward);
   rmPlaceObjectDefAtLoc(southernMineID, 0, 0.5, 0.5);
   for(i=0; <silverCount)
   {
      rmPlaceObjectDefAtLoc(southernMineID, 0, 0.5, 0.5);
   }
 
   // Place forests (North & South)
   int forestCount = -1;
   int failCount = -1;
   int forestTreeID = 0;
   if (mapSkin == 1) // snow
   {
      forestCount = (cNumberNonGaiaPlayers * 4) + rmRandInt(0,cNumberNonGaiaPlayers);
   }
   else if (mapSkin == 2) // grass
   {
      forestCount = (cNumberNonGaiaPlayers * 5) + rmRandInt(0,cNumberNonGaiaPlayers);
   }
   else // desert
   {
      forestCount = (cNumberNonGaiaPlayers * 3) - rmRandInt(0,cNumberNonGaiaPlayers);
   }
   failCount=0;
   for (i=0; <forestCount)  // Northern forests
   {   
      int northForest = rmCreateArea("northforest"+i);
      rmSetAreaWarnFailure(northForest, false);
      if (mapSkin == 1) // snow
      {
         rmSetAreaSize(northForest, rmAreaTilesToFraction(200), rmAreaTilesToFraction(400));
         rmSetAreaForestType(northForest, "great lakes forest snow");
         rmSetAreaForestDensity(northForest, 1.0);

      }
      else if (mapSkin == 2) // grass
      {
         rmSetAreaSize(northForest, rmAreaTilesToFraction(250), rmAreaTilesToFraction(400));
         rmSetAreaForestType(northForest, "great lakes forest");
         rmSetAreaForestDensity(northForest, 0.8);
      }
      else // desert
      {
         rmSetAreaSize(northForest, rmAreaTilesToFraction(150), rmAreaTilesToFraction(250));
         rmSetAreaForestType(northForest, "texas forest");
         rmSetAreaForestDensity(northForest, 0.7);
      }
      rmAddAreaToClass(northForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(northForest, 0.5);
      rmSetAreaForestUnderbrush(northForest, 0.9);
      rmSetAreaBaseHeight(northForest, 2.0);
      rmSetAreaMinBlobs(northForest, 1);
      rmSetAreaMaxBlobs(northForest, 3);
      rmSetAreaMinBlobDistance(northForest, 16.0);
      rmSetAreaMaxBlobDistance(northForest, 30.0);
      rmSetAreaCoherence(northForest, 0.7);
      rmSetAreaSmoothDistance(northForest, 1);
      rmAddAreaConstraint(northForest, avoidImportantItem);
      rmAddAreaConstraint(northForest, avoidTradeRoute);
      rmAddAreaConstraint(northForest, avoidTradeSockets);
      rmAddAreaConstraint(northForest, avoidNatives);
      rmAddAreaConstraint(northForest, avoidLakesShort);
      rmAddAreaConstraint(northForest, shortAvoidCoin);
      rmAddAreaConstraint(northForest, shortPlayerConstraint);
      rmAddAreaConstraint(northForest, forestConstraint);
      rmAddAreaConstraint(northForest, Northward);
      if(rmBuildArea(northForest)==false)
      {
         // Stop trying once we fail 5 times in a row.  
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount=0;
      }
   }
   failCount=0;
   for (i=0; <forestCount)  // Southern forests
   {   
      int southForest = rmCreateArea("southforest"+i);
      rmSetAreaWarnFailure(southForest, false);
      if (mapSkin == 1) // snow
      {
         rmSetAreaSize(southForest, rmAreaTilesToFraction(200), rmAreaTilesToFraction(400));
         rmSetAreaForestType(southForest, "great lakes forest snow");
         rmSetAreaForestDensity(southForest, 1.0);
      }
      else if (mapSkin == 2) // grass
      {
         rmSetAreaSize(southForest, rmAreaTilesToFraction(250), rmAreaTilesToFraction(400));
         rmSetAreaForestType(southForest, "great lakes forest");
         rmSetAreaForestDensity(southForest, 0.8);
      }
      else // desert
      {
         rmSetAreaSize(southForest, rmAreaTilesToFraction(150), rmAreaTilesToFraction(250));
         rmSetAreaForestType(southForest, "texas forest");
         rmSetAreaForestDensity(southForest, 0.7);
      }
      rmAddAreaToClass(southForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(southForest, 0.5);
      rmSetAreaForestUnderbrush(southForest, 0.9);
      rmSetAreaBaseHeight(southForest, 2.0);
      rmSetAreaMinBlobs(southForest, 1);
      rmSetAreaMaxBlobs(southForest, 3);
      rmSetAreaMinBlobDistance(southForest, 16.0);
      rmSetAreaMaxBlobDistance(southForest, 30.0);
      rmSetAreaCoherence(southForest, 0.7);
      rmSetAreaSmoothDistance(southForest, 1);
      rmAddAreaConstraint(southForest, avoidImportantItem);
      rmAddAreaConstraint(southForest, avoidTradeRoute);
      rmAddAreaConstraint(southForest, avoidTradeSockets);
      rmAddAreaConstraint(southForest, avoidNatives);
      rmAddAreaConstraint(southForest, avoidLakesShort);
      rmAddAreaConstraint(southForest, shortAvoidCoin);
      rmAddAreaConstraint(southForest, shortPlayerConstraint);
      rmAddAreaConstraint(southForest, forestConstraint);
      rmAddAreaConstraint(southForest, Southward);
      if(rmBuildArea(southForest)==false)
      {
         // Stop trying once we fail 5 times in a row.  
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount=0;
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.60);


   // Place additional food sources

   // Snow: place musk oxen and caribous
   if (mapSkin == 1)
   {
      // Place musk oxen
      int muskoxCount = rmRandInt(3,4);
      int muskoxHerdCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int muskoxNorthID = rmCreateObjectDef("musk ox herd north");
      rmAddObjectDefItem(muskoxNorthID, "muskOx", muskoxCount, 5.0);
      rmSetObjectDefMinDistance(muskoxNorthID, 0.0);
      rmSetObjectDefMaxDistance(muskoxNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(muskoxNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(muskoxNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(muskoxNorthID, playerConstraint);
      rmAddObjectDefConstraint(muskoxNorthID, muskoxConstraint);
      rmAddObjectDefConstraint(muskoxNorthID, Northward);
      rmSetObjectDefCreateHerd(muskoxNorthID, true);
      for(i=0; <muskoxHerdCount)
      { 
         rmPlaceObjectDefAtLoc(muskoxNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int muskoxSouthID = rmCreateObjectDef("musk ox herd south");
      rmAddObjectDefItem(muskoxSouthID, "muskOx", muskoxCount, 5.0);
      rmSetObjectDefMinDistance(muskoxSouthID, 0.0);
      rmSetObjectDefMaxDistance(muskoxSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(muskoxSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(muskoxSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(muskoxSouthID, playerConstraint);
      rmAddObjectDefConstraint(muskoxSouthID, muskoxConstraint);
      rmAddObjectDefConstraint(muskoxSouthID, Southward);
      rmSetObjectDefCreateHerd(muskoxSouthID, true);
      for(i=0; <muskoxHerdCount)
      { 
         rmPlaceObjectDefAtLoc(muskoxSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place caribous
      int caribouCount = rmRandInt(6,7);
      int caribouHerdCount = 2*cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int caribouNorthID = rmCreateObjectDef("caribou herd north");
      rmAddObjectDefItem(caribouNorthID, "caribou", caribouCount, 5.0);
      rmSetObjectDefMinDistance(caribouNorthID, 0.0);
      rmSetObjectDefMaxDistance(caribouNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(caribouNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(caribouNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(caribouNorthID, playerConstraint);
      rmAddObjectDefConstraint(caribouNorthID, muskoxConstraint);
      rmAddObjectDefConstraint(caribouNorthID, caribouConstraint);
      rmAddObjectDefConstraint(caribouNorthID, Northward);
      rmSetObjectDefCreateHerd(caribouNorthID, true);
      for(i=0; <caribouHerdCount)
      { 
         rmPlaceObjectDefAtLoc(caribouNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int caribouSouthID = rmCreateObjectDef("caribou herd south");
      rmAddObjectDefItem(caribouSouthID, "caribou", caribouCount, 5.0);
      rmSetObjectDefMinDistance(caribouSouthID, 0.0);
      rmSetObjectDefMaxDistance(caribouSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(caribouSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(caribouSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(caribouSouthID, playerConstraint);
      rmAddObjectDefConstraint(caribouSouthID, muskoxConstraint);
      rmAddObjectDefConstraint(caribouSouthID, caribouConstraint);
      rmAddObjectDefConstraint(caribouSouthID, Southward);
      rmSetObjectDefCreateHerd(caribouSouthID, true);
      for(i=0; <caribouHerdCount)
      { 
         rmPlaceObjectDefAtLoc(caribouSouthID, 0, 0.5, 0.5); // southern half of the map
      }
   }
   // Grass: place elks, deer, sheep and berry bushes
   else if (mapSkin == 2)
   {
      // Place elks
      int elkCount = rmRandInt(3,4);
      int elkHerdCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int elkNorthID = rmCreateObjectDef("elk herd north");
      rmAddObjectDefItem(elkNorthID, "elk", elkCount, 5.0);
      rmSetObjectDefMinDistance(elkNorthID, 0.0);
      rmSetObjectDefMaxDistance(elkNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(elkNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(elkNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(elkNorthID, playerConstraint);
      rmAddObjectDefConstraint(elkNorthID, elkConstraint);
      rmAddObjectDefConstraint(elkNorthID, Northward);
      rmSetObjectDefCreateHerd(elkNorthID, true);
      for(i=0; <elkHerdCount)
      { 
         rmPlaceObjectDefAtLoc(elkNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int elkSouthID = rmCreateObjectDef("elk herd south");
      rmAddObjectDefItem(elkSouthID, "elk", elkCount, 5.0);
      rmSetObjectDefMinDistance(elkSouthID, 0.0);
      rmSetObjectDefMaxDistance(elkSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(elkSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(elkSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(elkSouthID, playerConstraint);
      rmAddObjectDefConstraint(elkSouthID, elkConstraint);
      rmAddObjectDefConstraint(elkSouthID, Southward);
      rmSetObjectDefCreateHerd(elkSouthID, true);
      for(i=0; <elkHerdCount)
      { 
         rmPlaceObjectDefAtLoc(elkSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place deer
      int deerCount = rmRandInt(7,9);
      int deerHerdCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int deerNorthID = rmCreateObjectDef("deer herd north");
      rmAddObjectDefItem(deerNorthID, "deer", deerCount, 5.0);
      rmSetObjectDefMinDistance(deerNorthID, 0.0);
      rmSetObjectDefMaxDistance(deerNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(deerNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(deerNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(deerNorthID, playerConstraint);
      rmAddObjectDefConstraint(deerNorthID, elkConstraint);
      rmAddObjectDefConstraint(deerNorthID, deerConstraint);
      rmAddObjectDefConstraint(deerNorthID, Northward);
      rmSetObjectDefCreateHerd(deerNorthID, true);
      for(i=0; <deerHerdCount)
      { 
         rmPlaceObjectDefAtLoc(deerNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int deerSouthID = rmCreateObjectDef("deer herd south");
      rmAddObjectDefItem(deerSouthID, "deer", deerCount, 5.0);
      rmSetObjectDefMinDistance(deerSouthID, 0.0);
      rmSetObjectDefMaxDistance(deerSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(deerSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(deerSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(deerSouthID, playerConstraint);
      rmAddObjectDefConstraint(deerSouthID, elkConstraint);
      rmAddObjectDefConstraint(deerSouthID, deerConstraint);
      rmAddObjectDefConstraint(deerSouthID, Southward);
      rmSetObjectDefCreateHerd(deerSouthID, true);
      for(i=0; <deerHerdCount)
      { 
         rmPlaceObjectDefAtLoc(deerSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place sheep
      int sheepCount = rmRandInt(2,3);
      int sheepHerdCount = rmRandInt(0,cNumberNonGaiaPlayers) + rmRandInt(0,cNumberNonGaiaPlayers);
      if (sheepHerdCount == 1)
      {
         sheepHerdCount = 2; // guarantees a minimum number of sheep if there are any at all
      }
      int sheepNorthID = rmCreateObjectDef("sheep herd north");
      rmAddObjectDefItem(sheepNorthID, "sheep", sheepCount, 5.0);
      rmSetObjectDefMinDistance(sheepNorthID, 0.0);
      rmSetObjectDefMaxDistance(sheepNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(sheepNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(sheepNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(sheepNorthID, playerConstraint);
      rmAddObjectDefConstraint(sheepNorthID, elkConstraint);
      rmAddObjectDefConstraint(sheepNorthID, deerConstraint);
      rmAddObjectDefConstraint(sheepNorthID, sheepConstraint);
      rmAddObjectDefConstraint(sheepNorthID, Northward);
      for(i=0; <sheepHerdCount)
      { 
         rmPlaceObjectDefAtLoc(sheepNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int sheepSouthID = rmCreateObjectDef("sheep herd south");
      rmAddObjectDefItem(sheepSouthID, "sheep", sheepCount, 5.0);
      rmSetObjectDefMinDistance(sheepSouthID, 0.0);
      rmSetObjectDefMaxDistance(sheepSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(sheepSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(sheepSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(sheepSouthID, playerConstraint);
      rmAddObjectDefConstraint(sheepSouthID, elkConstraint);
      rmAddObjectDefConstraint(sheepSouthID, deerConstraint);
      rmAddObjectDefConstraint(sheepSouthID, sheepConstraint);
      rmAddObjectDefConstraint(sheepSouthID, Southward);
      for(i=0; <sheepHerdCount)
      { 
         rmPlaceObjectDefAtLoc(sheepSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place berry bushes
      int berryCount = rmRandInt(2,4);
      int berryBushCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int berryNorthID = rmCreateObjectDef("berry bushes north");
      rmAddObjectDefItem(berryNorthID, "berrybush", berryCount, 2.0);
      rmSetObjectDefMinDistance(berryNorthID, 0.0);
      rmSetObjectDefMaxDistance(berryNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(berryNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(berryNorthID, avoidTradeRouteFar);
      rmAddObjectDefConstraint(berryNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(berryNorthID, playerConstraint);
      rmAddObjectDefConstraint(berryNorthID, elkConstraint);
      rmAddObjectDefConstraint(berryNorthID, deerConstraint);
      rmAddObjectDefConstraint(berryNorthID, sheepConstraint);
      rmAddObjectDefConstraint(berryNorthID, berryConstraint);
      rmAddObjectDefConstraint(berryNorthID, Northward);
      for(i=0; <berryBushCount)
      { 
         rmPlaceObjectDefAtLoc(berryNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int berrySouthID = rmCreateObjectDef("berry bushes south");
      rmAddObjectDefItem(berrySouthID, "berrybush", berryCount, 2.0);
      rmSetObjectDefMinDistance(berrySouthID, 0.0);
      rmSetObjectDefMaxDistance(berrySouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(berrySouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(berrySouthID, avoidTradeRouteFar);
      rmAddObjectDefConstraint(berrySouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(berrySouthID, playerConstraint);
      rmAddObjectDefConstraint(berrySouthID, elkConstraint);
      rmAddObjectDefConstraint(berrySouthID, deerConstraint);
      rmAddObjectDefConstraint(berrySouthID, sheepConstraint);
      rmAddObjectDefConstraint(berrySouthID, berryConstraint);
      rmAddObjectDefConstraint(berrySouthID, Southward);
      for(i=0; <berryBushCount)
      { 
         rmPlaceObjectDefAtLoc(berrySouthID, 0, 0.5, 0.5); // southern half of the map
      }
   }
   // Desert: place bisons, pronghorns, cows and berry bushes
   else // mapSkin == 3
   {
      // Place bisons
      int bisonCount = rmRandInt(4,5);
      int bisonHerdCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int bisonNorthID = rmCreateObjectDef("bison herd north");
      rmAddObjectDefItem(bisonNorthID, "bison", bisonCount, 5.0);
      rmSetObjectDefMinDistance(bisonNorthID, 0.0);
      rmSetObjectDefMaxDistance(bisonNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(bisonNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(bisonNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(bisonNorthID, playerConstraint);
      rmAddObjectDefConstraint(bisonNorthID, bisonConstraint);
      rmAddObjectDefConstraint(bisonNorthID, Northward);
      rmSetObjectDefCreateHerd(bisonNorthID, true);
      for(i=0; <bisonHerdCount)
      { 
         rmPlaceObjectDefAtLoc(bisonNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int bisonSouthID = rmCreateObjectDef("bison herd south");
      rmAddObjectDefItem(bisonSouthID, "bison", bisonCount, 5.0);
      rmSetObjectDefMinDistance(bisonSouthID, 0.0);
      rmSetObjectDefMaxDistance(bisonSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(bisonSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(bisonSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(bisonSouthID, playerConstraint);
      rmAddObjectDefConstraint(bisonSouthID, bisonConstraint);
      rmAddObjectDefConstraint(bisonSouthID, Southward);
      rmSetObjectDefCreateHerd(bisonSouthID, true);
      for(i=0; <bisonHerdCount)
      { 
         rmPlaceObjectDefAtLoc(bisonSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place pronghorns
      int pronghornCount = rmRandInt(4,6);
      int pronghornHerdCount = cNumberNonGaiaPlayers + rmRandInt(0,cNumberNonGaiaPlayers);
      int pronghornNorthID = rmCreateObjectDef("pronghorn herd north");
      rmAddObjectDefItem(pronghornNorthID, "pronghorn", pronghornCount, 5.0);
      rmSetObjectDefMinDistance(pronghornNorthID, 0.0);
      rmSetObjectDefMaxDistance(pronghornNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(pronghornNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(pronghornNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(pronghornNorthID, playerConstraint);
      rmAddObjectDefConstraint(pronghornNorthID, bisonConstraint);
      rmAddObjectDefConstraint(pronghornNorthID, pronghornConstraint);
      rmAddObjectDefConstraint(pronghornNorthID, Northward);
      rmSetObjectDefCreateHerd(pronghornNorthID, true);
      for(i=0; <pronghornHerdCount)
      { 
         rmPlaceObjectDefAtLoc(pronghornNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int pronghornSouthID = rmCreateObjectDef("pronghorn herd south");
      rmAddObjectDefItem(pronghornSouthID, "pronghorn", pronghornCount, 5.0);
      rmSetObjectDefMinDistance(pronghornSouthID, 0.0);
      rmSetObjectDefMaxDistance(pronghornSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(pronghornSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(pronghornSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(pronghornSouthID, playerConstraint);
      rmAddObjectDefConstraint(pronghornSouthID, bisonConstraint);
      rmAddObjectDefConstraint(pronghornSouthID, pronghornConstraint);
      rmAddObjectDefConstraint(pronghornSouthID, Southward);
      rmSetObjectDefCreateHerd(pronghornSouthID, true);
      for(i=0; <pronghornHerdCount)
      { 
         rmPlaceObjectDefAtLoc(pronghornSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place cows
      int cowCount = 2;
      int cowHerdCount = rmRandInt(0,cNumberNonGaiaPlayers);
      if (cowHerdCount == 1)
      {
         cowHerdCount = 2; // guarantees a minimum number of cows if there are any at all
      }
      int cowNorthID = rmCreateObjectDef("cow herd north");
      rmAddObjectDefItem(cowNorthID, "cow", cowCount, 15.0);
      rmSetObjectDefMinDistance(cowNorthID, 0.0);
      rmSetObjectDefMaxDistance(cowNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(cowNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(cowNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(cowNorthID, playerConstraint);
      rmAddObjectDefConstraint(cowNorthID, bisonConstraint);
      rmAddObjectDefConstraint(cowNorthID, pronghornConstraint);
      rmAddObjectDefConstraint(cowNorthID, cowConstraint);
      rmAddObjectDefConstraint(cowNorthID, Northward);
      for(i=0; <cowHerdCount)
      { 
         rmPlaceObjectDefAtLoc(cowNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int cowSouthID = rmCreateObjectDef("cow herd south");
      rmAddObjectDefItem(cowSouthID, "cow", cowCount, 15.0);
      rmSetObjectDefMinDistance(cowSouthID, 0.0);
      rmSetObjectDefMaxDistance(cowSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(cowSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(cowSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(cowSouthID, playerConstraint);
      rmAddObjectDefConstraint(cowSouthID, bisonConstraint);
      rmAddObjectDefConstraint(cowSouthID, pronghornConstraint);
      rmAddObjectDefConstraint(cowSouthID, cowConstraint);
      rmAddObjectDefConstraint(cowSouthID, Southward);
      for(i=0; <cowHerdCount)
      { 
         rmPlaceObjectDefAtLoc(cowSouthID, 0, 0.5, 0.5); // southern half of the map
      }
      // Place berry bushes
      int berryCountDesert = 2;
      int berryBushCountDesert = rmRandInt(2,cNumberNonGaiaPlayers);
      int berryDesertNorthID = rmCreateObjectDef("berry bushes desert north");
      rmAddObjectDefItem(berryDesertNorthID, "berrybush", berryCountDesert, 2.0);
      rmSetObjectDefMinDistance(berryDesertNorthID, 0.0);
      rmSetObjectDefMaxDistance(berryDesertNorthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(berryDesertNorthID, avoidImpassableLand);
      rmAddObjectDefConstraint(berryDesertNorthID, avoidTradeRouteFar);
      rmAddObjectDefConstraint(berryDesertNorthID, shortAvoidCoin);
      rmAddObjectDefConstraint(berryDesertNorthID, playerConstraint);
      rmAddObjectDefConstraint(berryDesertNorthID, bisonConstraint);
      rmAddObjectDefConstraint(berryDesertNorthID, pronghornConstraint);
      rmAddObjectDefConstraint(berryDesertNorthID, cowConstraint);
      rmAddObjectDefConstraint(berryDesertNorthID, berryConstraint);
      rmAddObjectDefConstraint(berryDesertNorthID, Northward);
      for(i=0; <berryBushCountDesert)
      { 
         rmPlaceObjectDefAtLoc(berryDesertNorthID, 0, 0.5, 0.5); // northern half of the map
      }
      int berryDesertSouthID = rmCreateObjectDef("berry bushes desert south");
      rmAddObjectDefItem(berryDesertSouthID, "berrybush", berryCountDesert, 2.0);
      rmSetObjectDefMinDistance(berryDesertSouthID, 0.0);
      rmSetObjectDefMaxDistance(berryDesertSouthID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(berryDesertSouthID, avoidImpassableLand);
      rmAddObjectDefConstraint(berryDesertSouthID, avoidTradeRouteFar);
      rmAddObjectDefConstraint(berryDesertSouthID, shortAvoidCoin);
      rmAddObjectDefConstraint(berryDesertSouthID, playerConstraint);
      rmAddObjectDefConstraint(berryDesertSouthID, bisonConstraint);
      rmAddObjectDefConstraint(berryDesertSouthID, pronghornConstraint);
      rmAddObjectDefConstraint(berryDesertSouthID, cowConstraint);
      rmAddObjectDefConstraint(berryDesertSouthID, berryConstraint);
      rmAddObjectDefConstraint(berryDesertSouthID, Southward);
      for(i=0; <berryBushCountDesert)
      { 
         rmPlaceObjectDefAtLoc(berryDesertSouthID, 0, 0.5, 0.5); // southern half of the map
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.70);


   // Define and place nuggets

   // Easy nuggets (0-3 per player, close to the TC)
   int nugget1 = rmCreateObjectDef("nugget easy"); 
   rmAddObjectDefItem(nugget1, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(1, 1);
   rmAddObjectDefToClass(nugget1, rmClassID("nuggets"));
   rmAddObjectDefConstraint(nugget1, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget1, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget1, avoidNuggets);
   rmAddObjectDefConstraint(nugget1, avoidTradeSockets);
   rmAddObjectDefConstraint(nugget1, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget1, avoidAll);
   rmAddObjectDefConstraint(nugget1, circleConstraint);
   rmSetObjectDefMinDistance(nugget1, 30.0);
   rmSetObjectDefMaxDistance(nugget1, 50.0);
   rmPlaceObjectDefPerPlayer(nugget1, false, rmRandInt(0,3));

   // Medium nuggets (0-2 per player, further from the TC)
   int nugget2 = rmCreateObjectDef("nugget medium"); 
   rmAddObjectDefItem(nugget2, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(2, 2);
   rmAddObjectDefToClass(nugget2, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget2, 0.0);
   rmSetObjectDefMaxDistance(nugget2, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(nugget2, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget2, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget2, avoidNuggets);
   rmAddObjectDefConstraint(nugget2, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget2, circleConstraint);
   rmAddObjectDefConstraint(nugget2, avoidAll);
   rmSetObjectDefMinDistance(nugget2, 50.0);
   rmSetObjectDefMaxDistance(nugget2, 100.0);
   rmPlaceObjectDefPerPlayer(nugget2, false, rmRandInt(0,2));

   // Hard nuggets (up to 2 per player, central part of the map)
   int nugget3 = rmCreateObjectDef("nugget hard"); 
   rmAddObjectDefItem(nugget3, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(3, 3);
   rmAddObjectDefToClass(nugget3, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget3, 0.15);
   rmSetObjectDefMaxDistance(nugget3, rmXFractionToMeters(0.35));
   rmAddObjectDefConstraint(nugget3, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget3, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget3, avoidNuggets);
   rmAddObjectDefConstraint(nugget3, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget3, circleConstraint);
   rmAddObjectDefConstraint(nugget3, avoidAll);
   rmPlaceObjectDefAtLoc(nugget3, 0, 0.5, 0.5, rmRandInt(2,(2*cNumberNonGaiaPlayers)));

   // Very hard nuggets (0-2 in total, central part of the map)
   int nugget4 = rmCreateObjectDef("nugget nuts"); 
   rmAddObjectDefItem(nugget4, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(4, 4);
   rmAddObjectDefToClass(nugget4, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget4, 0.15);
   rmSetObjectDefMaxDistance(nugget4, rmXFractionToMeters(0.25));
   rmAddObjectDefConstraint(nugget4, longPlayerConstraint);
   rmAddObjectDefConstraint(nugget4, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget4, avoidNuggets);
   rmAddObjectDefConstraint(nugget4, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget4, circleConstraint);
   rmAddObjectDefConstraint(nugget4, avoidAll);
   rmPlaceObjectDefAtLoc(nugget4, 0, 0.5, 0.5, rmRandInt(0,2));

   // Map generation status bar
   rmSetStatusText("",0.80);


   // Add decoration

   // Place single trees
   int loneTree = rmCreateObjectDef("lone trees");
   int numTrees = size/10;
   if (mapSkin == 1) // snow
   {
      numTrees = size/15;
      rmAddObjectDefItem(loneTree, "TreeGreatLakesSnow", 1, 0.0);
   }
   else if (mapSkin == 2) // grass
   {
      numTrees = size/10;
      rmAddObjectDefItem(loneTree, "TreeGreatPlains", 1, 0.0);
   }
   else // desert
   {
      numTrees = size/15;
      rmAddObjectDefItem(loneTree, "TreeTexasDirt", 1, 0.0);
   }
   rmSetObjectDefMinDistance(loneTree, 0.0);
   rmSetObjectDefMaxDistance(loneTree, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(loneTree, shortPlayerConstraint);
   rmAddObjectDefConstraint(loneTree, avoidTradeRoute);
   rmAddObjectDefConstraint(loneTree, avoidAll);
   rmPlaceObjectDefAtLoc(loneTree, 0, 0.5, 0.5, numTrees);

   // Place dirt or grass patches
   if (mapSkin == 1) // snow
   {
      int numDirtPatch = size/10;
      for (i=0; <numDirtPatch)
      {   
         int dirtPatch = rmCreateArea("dirtpatch"+i);
         rmSetAreaWarnFailure(dirtPatch, false);
         rmSetAreaSize(dirtPatch, rmAreaTilesToFraction(20), rmAreaTilesToFraction(40));
         rmSetAreaTerrainType(dirtPatch, "yukon\ground8_yuk");
         rmSetAreaCoherence(dirtPatch, 0.7);
         rmSetAreaSmoothDistance(dirtPatch, 1);
         rmAddAreaConstraint(dirtPatch, avoidImpassableLand);
         rmAddAreaConstraint(dirtPatch, forestConstraintShort);
         rmBuildArea(dirtPatch);
      }
   }
   else // grass or desert
   {
      int grassPatchGroupType = rmRandInt(1,7);
      int grassPatchGroup = rmCreateGrouping("grass patch group"+i, "gp_grasspatch0"+grassPatchGroupType);
      rmSetGroupingMinDistance(grassPatchGroup, 0.0);
      rmSetGroupingMaxDistance(grassPatchGroup, rmXFractionToMeters(0.50));
      rmAddGroupingConstraint(grassPatchGroup, playerConstraint);
      rmAddGroupingConstraint(grassPatchGroup, avoidNatives);
      rmAddGroupingConstraint(grassPatchGroup, avoidTradeRoute);
      rmAddGroupingConstraint(grassPatchGroup, avoidTradeSockets);
      rmAddGroupingConstraint(grassPatchGroup, avoidAll);
      rmAddGroupingConstraint(grassPatchGroup, avoidImpassableLand);
      rmPlaceGroupingAtLoc(grassPatchGroup, 0, 0.5, 0.5, size/20);
   }

   // Place perching vultures and big rocks
   if (mapSkin == 3) // desert
   {
      int vultureCount = rmRandInt(2,6);
      int vultureID = rmCreateObjectDef("perching vultures");
      rmAddObjectDefItem(vultureID, "PropVulturePerching", 1, 0.0);
      rmSetObjectDefMinDistance(vultureID, 0.0);
      rmSetObjectDefMaxDistance(vultureID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(vultureID, extremePlayerConstraint);
      rmAddObjectDefConstraint(vultureID, avoidTradeRoute);
      rmAddObjectDefConstraint(vultureID, avoidTradeSockets);
      rmAddObjectDefConstraint(vultureID, avoidVultures);
      rmAddObjectDefConstraint(vultureID, avoidAll);
      rmAddObjectDefConstraint(vultureID, avoidImpassableLand);
      rmPlaceObjectDefAtLoc(vultureID, 0, 0.5, 0.5, vultureCount);
   }

   // Place rocks
   if (mapSkin == 3) // desert
   {
      int rockCount = rmRandInt(2,6);
      int rockID = rmCreateObjectDef("big rock");
      rmAddObjectDefItem(rockID, "BigPropTexas", 1, 0.0);
      rmSetObjectDefMinDistance(rockID, 0.0);
      rmSetObjectDefMaxDistance(rockID, rmXFractionToMeters(0.50));
      rmAddObjectDefConstraint(rockID, longPlayerConstraint);
      rmAddObjectDefConstraint(rockID, avoidTradeRoute);
      rmAddObjectDefConstraint(rockID, avoidTradeSockets);
      rmAddObjectDefConstraint(rockID, avoidRock);
      rmAddObjectDefConstraint(rockID, avoidVultures);
      rmAddObjectDefConstraint(rockID, avoidAll);
      rmAddObjectDefConstraint(rockID, avoidImpassableLand);
      rmPlaceObjectDefAtLoc(rockID, 0, 0.5, 0.5, rockCount);
   }

   // Place bison carcasses
   if (mapSkin > 1) // grass or desert
   {
      int carcassCount = rmRandInt(1,3);
      int bisonCarcass = rmCreateGrouping("bison carcass", "gp_carcass_bison");
      rmSetGroupingMinDistance(bisonCarcass, 0.0);
      rmSetGroupingMaxDistance(bisonCarcass, rmXFractionToMeters(0.50));
      rmAddGroupingConstraint(bisonCarcass, extremePlayerConstraint);
      rmAddGroupingConstraint(bisonCarcass, avoidNatives);
      rmAddGroupingConstraint(bisonCarcass, avoidTradeRoute);
      rmAddGroupingConstraint(bisonCarcass, avoidTradeSockets);
      rmAddGroupingConstraint(bisonCarcass, avoidVultures);
      rmAddGroupingConstraint(bisonCarcass, avoidRock);
      rmAddGroupingConstraint(bisonCarcass, avoidAll);
      rmAddGroupingConstraint(bisonCarcass, avoidImpassableLand);
      rmPlaceGroupingAtLoc(bisonCarcass, 0, 0.5, 0.5, carcassCount);
   }

   // Place geysers
   if (mapSkin == 2) // grass
   {
      int geyserCount = rmRandInt(1,3);
      int geyserID = rmCreateGrouping("geyser", "prop_geyser");
      rmSetGroupingMinDistance(geyserID, 0.0);
      rmSetGroupingMaxDistance(geyserID, rmXFractionToMeters(0.50));
      rmAddGroupingConstraint(geyserID, longPlayerConstraint);
      rmAddGroupingConstraint(geyserID, avoidTradeRoute);
      rmAddGroupingConstraint(geyserID, avoidTradeSockets);
      rmAddGroupingConstraint(geyserID, avoidVultures);
      rmAddGroupingConstraint(geyserID, avoidAll);
      rmAddGroupingConstraint(geyserID, avoidImpassableLand);
      rmPlaceGroupingAtLoc(geyserID, 0, 0.5, 0.5, geyserCount);
   }

   // Map generation status bar
   rmSetStatusText("",0.90);


   // Define triggers for King of the Hill style game

   // Get number of trading posts
   int numTradingPosts = rmGetNumberUnitsPlaced(socketID);

   // Init quest vars
   rmCreateTrigger("InitQuestVar");
   rmSwitchToTrigger(rmTriggerID("InitQuestVar"));
   rmSetTriggerActive(true);
   rmSetTriggerLoop(false);
   rmSetTriggerRunImmediately(true);
   rmSetTriggerPriority(4);
   for (i = 1; <=cNumberNonGaiaPlayers)
   {
      rmAddTriggerEffect("Quest Var Set");
      rmSetTriggerEffectParam("QVName", "CounterActive"+i);
      rmSetTriggerEffectParamInt("Value", 0);
   }

   // Initial message
   rmCreateTrigger("KotHMessage");
   rmSwitchToTrigger(rmTriggerID("KotHMessage"));
   rmSetTriggerActive(true);
   rmSetTriggerLoop(false);
   rmSetTriggerRunImmediately(true);
   rmSetTriggerPriority(4);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 2);
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", " ");
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", " ");
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", "This game is played King of the Hill style:");
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", "- Capture all trading posts to start victory counter");
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", "- Destroy two trading posts to stop victory counter");
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", " ");
   rmAddTriggerEffect("Send Chat");
   rmSetTriggerEffectParamInt("PlayerID", 0);
   rmSetTriggerEffectParam("Message", " ");

   // Counters & victory
   for (i = 1; <=cNumberNonGaiaPlayers)
   {
      //Create all triggers first
      rmCreateTrigger("StartCounter"+i);
      rmCreateTrigger("StopCounter"+i);
      rmCreateTrigger("Victory"+i);

      // Define triggers
      rmSwitchToTrigger(rmTriggerID("StartCounter"+i));
      rmSetTriggerActive(true);
      rmSetTriggerLoop(true);
      rmSetTriggerRunImmediately(false);
      rmSetTriggerPriority(4);
      rmAddTriggerCondition("Player Unit Count");
      rmSetTriggerConditionParamInt("PlayerID", i);
      rmSetTriggerConditionParam("ProtoUnit", "TradingPost");
      rmSetTriggerConditionParam("Op", "==");
      rmSetTriggerConditionParamInt("Count", numTradingPosts);
      rmAddTriggerCondition("Quest Var Check");
      rmSetTriggerConditionParam("QuestVar", "CounterActive"+i);
      rmSetTriggerConditionParam("Op", "==");
      rmSetTriggerConditionParamInt("Value", 0);
      rmAddTriggerEffect("Counter:Add Timer");
      rmSetTriggerEffectParam("Name", "Countdown"+i);
      rmSetTriggerEffectParamInt("Start", 600);
      rmSetTriggerEffectParamInt("Stop", 0);
      rmSetTriggerEffectParam("Msg", "A new King will be crowned in" );
      rmSetTriggerEffectParamInt("Event", rmTriggerID("Victory"+i));
      rmAddTriggerEffect("Send Chat");
      rmSetTriggerEffectParamInt("PlayerID", 0);
      rmSetTriggerEffectParam("Message", "All trading posts have been captured and the victory counter started!");
      rmAddTriggerEffect("Quest Var Set");
      rmSetTriggerEffectParam("QVName", "CounterActive"+i);
      rmSetTriggerEffectParamInt("Value", 1);

      rmSwitchToTrigger(rmTriggerID("StopCounter"+i));
      rmSetTriggerActive(true);
      rmSetTriggerLoop(true);
      rmSetTriggerRunImmediately(false);
      rmSetTriggerPriority(4);
      rmAddTriggerCondition("Player Unit Count");
      rmSetTriggerConditionParamInt("PlayerID", i);
      rmSetTriggerConditionParam("ProtoUnit", "TradingPost");
      rmSetTriggerConditionParam("Op", "<");
      rmSetTriggerConditionParamInt("Count", (numTradingPosts - 1));
      rmAddTriggerCondition("Quest Var Check");
      rmSetTriggerConditionParam("QuestVar", "CounterActive"+i);
      rmSetTriggerConditionParam("Op", "==");
      rmSetTriggerConditionParamInt("Value", 1);
      rmAddTriggerEffect("Counter Stop");
      rmSetTriggerEffectParam("Name", "Countdown"+i);
      rmAddTriggerEffect("Send Chat");
      rmSetTriggerEffectParamInt("PlayerID", 0);
      rmSetTriggerEffectParam("Message", "Two trading posts have been destroyed and the victory counter stopped!");
      rmAddTriggerEffect("Quest Var Set");
      rmSetTriggerEffectParam("QVName", "CounterActive"+i);
      rmSetTriggerEffectParamInt("Value", 0);

      rmSwitchToTrigger(rmTriggerID("Victory"+i));
      rmSetTriggerActive(false);
      rmSetTriggerLoop(false);
      rmSetTriggerRunImmediately(false);
      rmSetTriggerPriority(4);
      rmAddTriggerEffect("Set Player Won");
      rmSetTriggerEffectParamInt("Player", i);
      rmAddTriggerEffect("End Game");
   }  

   // Map generation status bar
   rmSetStatusText("",1.0);
}  
